package com.example.venue_reservation_service.controller;


import com.example.venue_reservation_service.dto.QuestionDTO;
import com.example.venue_reservation_service.dto.QuestionQueryDTO;
import com.example.venue_reservation_service.dto.ReservationDTO;
import com.example.venue_reservation_service.service.QuestionService;
import com.example.venue_reservation_service.vo.Result;
import jakarta.annotation.Resource;
import org.springframework.web.bind.annotation.*;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/question")
@Tag(name = "提问管理", description = "用户提问及管理员审核相关接口")
@CrossOrigin
public class QuestionController {

    @Resource
    private QuestionService questionService;

    @Operation(summary = "创建提问", description = "用户提交新的提问内容",
            parameters = {
                    @Parameter(
                            name = "dto",
                            description = "用户账号，联系电话、疑问类型等信息",
                            required = true,
                            schema = @Schema(implementation = QuestionDTO.class)
                    )
            }
    )
    @PostMapping("/create")
    public Result create(@RequestBody QuestionDTO dto) {
        return questionService.create(dto);
    }

    @Operation(summary = "用户分页查询提问", description = "用户查询自己的提问列表，支持分页和筛选",
            parameters = {
                    @Parameter(
                            name = "dto",
                            description = "筛选条件：场馆、状态",
                            required = true,
                            schema = @Schema(implementation = QuestionQueryDTO.class)
                    )
            }
    )
    @PostMapping("/user/page")
    public Result userPage(@RequestBody QuestionQueryDTO dto) {
        return questionService.userPage(dto);
    }

    @Operation(summary = "管理员分页查询提问", description = "管理员查询所有用户的提问列表，支持分页和筛选",
            parameters = {
                    @Parameter(
                            name = "dto",
                            description = "提问查询DTO，包含用户ID、状态等筛选条件",
                            required = true,
                            schema = @Schema(implementation = ReservationDTO.class)
                    )
            }
    )
    @PostMapping("/admin/page")
    public Result adminPage(@RequestBody ReservationDTO dto) {
        return questionService.adminPage(dto);
    }
}