package com.example.venue_reservation_service.controller;

import com.example.venue_reservation_service.domain.VeMessage;
import com.example.venue_reservation_service.domain.VeSession;
import com.example.venue_reservation_service.service.VeMessageService;
import com.example.venue_reservation_service.service.VeSessionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jakarta.annotation.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/session")
@Api("ai会话管理模块")
@CrossOrigin
public class SessionController {

    @Resource
    private VeSessionService sessionService;

    @Resource
    private VeMessageService messageService;

    @ApiOperation("创建新会话")
    @PostMapping("/create")
    public ResponseEntity<VeSession> createSession(@RequestBody VeSession session) {
        VeSession savedSession = sessionService.saveSession(session);
        return ResponseEntity.status(HttpStatus.CREATED).body(savedSession);
    }

    @ApiOperation("获取用户会话历史")
    @GetMapping("/list")
    public ResponseEntity<List<VeSession>> getUserSessions(@RequestParam(name = "userId") Integer userId) {
        List<VeSession> sessions = sessionService.getSessionsByUserId(userId);
        return ResponseEntity.ok(sessions);
    }

    @ApiOperation("获取会话详情")
    @GetMapping("/{sessionId}")
    public ResponseEntity<VeSession> getSession(@PathVariable("sessionId") Integer sessionId) {
        VeSession session = sessionService.getById(sessionId);
        if (session != null) {
            return ResponseEntity.ok(session);
        } else {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }
    }

    @ApiOperation("获取指定会话的所有消息")
    @GetMapping("/message/{sessionId}")
    public ResponseEntity<List<VeMessage>> getSessionMessages(@PathVariable("sessionId") Integer sessionId) {
        List<VeMessage> messages = messageService.getMessagesBySessionId(sessionId);
        return ResponseEntity.ok(messages);
    }

    @ApiOperation("创建新消息")
    @PostMapping("/message/create")
    public ResponseEntity<VeMessage> createMessage(@RequestBody VeMessage message) {
        VeMessage savedMessage = messageService.saveMessage(message);
        return ResponseEntity.status(HttpStatus.CREATED).body(savedMessage);
    }
}
